"""
Pykrete test helpers
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

import os


def get_full_path(project, is_with_src_root=False):
    """Compose relative paths for tests

    :param project: simulated project
    :param is_with_src_root: should this repo's 'src' root be included
    :return: Composed path
    """
    root = 'pykrete'
    if is_with_src_root:
        root = os.path.join('src', root)
    return os.path.join(root, 'tests', 'src', project)



"""
Pykrete file tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import unittest
import logging
import os
from pykrete.io.file import File, ReReplacement, TextReplacement


class TestFile(unittest.TestCase):
    """Unit tests for the File and ReReplacement classes
    """
    _logger = logging.getLogger(__name__)
    _base_path = os.path.join('src', 'pykrete', 'tests', 'src')
    _temp_path = os.path.join(_base_path, 'temp.txt')

    def test_text_replacement(self):
        """Verify text replacement
        """
        contents = 'ROMANES EUNT DOMUS'
        expected = 'ROMANI ITE DOMUM'
        rep1 = TextReplacement('ROMANES', 'ROMANI')
        rep2 = TextReplacement('EUNT', 'ITE')
        rep3 = TextReplacement('DOMUS', 'DOMUM')
        self._logger.debug(contents)
        File(self._temp_path).write(contents)
        File(self._temp_path).update(lambda x: True, rep1, rep2, rep3)
        self._logger.debug(File(self._temp_path).read())
        self.assertEqual(expected, File(self._temp_path).read(), "text replacement failed")

    def test_regex_replacement(self):
        """Verify regex replacement
        """
        contents = 'I didn\'t expect the Spanish Inquisition!'
        expected = 'No one expects the Spanish Inquisition!'
        rep1 = ReReplacement(r'I .*?t', 'No one')
        rep2 = ReReplacement(r'e\S*t', 'expects')
        File(self._temp_path).write(contents)
        File(self._temp_path).update(lambda x: True, rep1, rep2)
        self.assertEqual(expected, File(self._temp_path).read(), "text replacement failed")

    def test_recurring_unchanged_replacement(self):
        """Verify that pairs of mirror changes leave the file as was
        """
        contents = File(os.path.join(self._base_path, 'AssemblyInfo.cs')).read()
        self._logger.debug(contents)
        rep_to = ReReplacement(r'"99.99.99.99"', '"1.2.3.4"')
        rep_from = ReReplacement(r'"1.2.3.4"', '"99.99.99.99"')
        File(self._temp_path).write(contents)
        for i in range(1, 10):
            self._logger.debug(i)
            File(self._temp_path).update(lambda x: True, rep_to, rep_from)
        self.assertEqual(contents, File(self._temp_path).read(), "contents differ")


if __name__ == '__main__':
    unittest.main()



"""
File IO classes
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from .common import write_lines
from .file import File
from .re_replacement import ReReplacement
from .text_replacement import TextReplacement


__all__ = ['write_lines', 'File', 'ReReplacement', 'TextReplacement']



"""
.Net source management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from .project_type import ProjectType
from .project import Project
from .solution import Solution

__all__ = ['ProjectType', 'Project', 'Solution']



"""
distutils pylint project-test command
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from .pylint_command import PylintCommand


class ProjectTestCommand(PylintCommand):
    """A custom command to run static-test on all python files in a package"""

    def _get_target_python_files_and_folders(self):
        """:return: The project's directory
        """
        return [self.path]



"""
GIT tag-based version bumper
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from .revision_type import RevisionType


class TagVersionBumper:
    """Handles version bump
    In the case of a release, a commit message containing '#major' will increment the major version,
     otherwise a commit message containing '#minor' will increment the minor version.
    Build is calculated as the current CI project-build# variable, minus a base build,
     or 0 if it is a release.
    """

    __major_part = 0
    __minor_part = 1
    __version_part_base = 0
    __version_bump_size = 1

    def __init__(self, repo, revision, base_build):
        """Initializes this instance from CI environment

        :param repo: project's repository (optional, defaults to git for the current folder)
        :param revision: Build revision
        :param base_build: base build for non-release versions
        """
        self._logger = logging.getLogger(__name__)
        self.__revision = revision
        self.__repo = repo
        self.__base_build = base_build
        self.__version_parts = []

    def advance_version(self, version_parts):
        """Advances the specified version

        :param version_parts: version to advance
        :return: advanced version
        """
        self.__version_parts = list(version_parts)
        if self.__revision.revision_type != RevisionType.Release:
            return self.__advance_build_relative_to_base_build()
        return self.__advance_major_minor_from_last_change_details()

    def __advance_build_relative_to_base_build(self):
        self.__version_parts[2] += self.__revision.build - self.__base_build
        self._logger.debug('Advanced version relative to base build: %s', self.__version_parts)
        return self.__version_parts

    def __advance_major_minor_from_last_change_details(self):
        change_details = self.__repo.get_head_change().details
        for key, value in {'#major': self.__major_part, '#minor': self.__minor_part}.items():
            if key in change_details:
                return self.__bump_version(value)
        return self.__advance_build_relative_to_base_build()

    def __bump_version(self, bump_part):
        self.__version_parts[bump_part] += self.__version_bump_size
        for i in range(bump_part + 1, len(self.__version_parts)):
            self.__version_parts[i] = self.__version_part_base
        self._logger.debug('Advanced version by bumping location %s: %s',
                           bump_part + 1, self.__version_parts)
        return self.__version_parts



"""
Repository management interface
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from abc import ABCMeta, abstractmethod


class Repo(metaclass=ABCMeta):
    """Repository interface"""

    @abstractmethod
    def get_all_tags(self):
        """Gets all tags from the current branch in the repo

        :return: A list of tag info tuples (name, message)
        """

    @abstractmethod
    def get_tags_from(self, pattern, is_must=False):
        """Gets all tags from the current branch in the repo, from the tag whose name matches the
         supplied pattern.

        :param pattern: name pattern to match
        :param is_must: True to raise an error if no match is found, False to return all tags in
        that case (with the first tuple as None).
        :return: A list of tag info tuples (name, message)
        """

    @abstractmethod
    def add_tag(self, name, message):
        """Adds a tag to the repo

        :param name: tag name
        :param message: tag message
        :return: the new tag's info tuple (name, message)
        """

    @abstractmethod
    def remove_tag(self, name, is_must):
        """Removes a tag from the repo

        :param name: tag name
        :param is_must: True to raise an error tag doesn't exist, False to just skip in that case.
        :exception: KeyError - tag not found
        """

    @abstractmethod
    def get_head_change(self, branch=None):
        """Gets the last change in the branch

        :param branch: branch name (optional, defaults to the current branch)
        :return: (pykrete.repo.Change) last change in the branch
        """



"""
Regex replacement class
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import os
from pykrete.calls import CheckedCall


class File:
    """Used to handle text files

    Properties:
    path (string): the file's path
    """

    def __init__(self, path):
        """Initialize this instance to handle the file at the specified path

        :param path: file path
        """
        self.path = path
        self.__contents = None

    def update(self, test, *replacements):
        """If the specified file's contents pass the test, preforms replacements and updates the
         file

        :param test: (function accepting a string and returning bool) a test on the file's contents
        :param replacements: (Replacement) replacement object(s)

        :return: (bool) True if test passed and file was updated, False if test failed (file won't
         be updated)
        """
        contents = self.__contents if self.__contents else self.read()
        if not test(contents):
            return False

        for replacement in replacements:
            contents = replacement.replace(contents)
        self.write(contents)

        if self.__contents:
            self.__contents = contents
        return True

    def write(self, contents):
        """Write contents to a file

        :param contents: (string) contents to be written
        """
        self.__write(lambda file: file.write(contents))

    def write_doc(self, doc):
        """Write XML document to a file

        :param doc: (Document) XML to be written
        """
        self.__write(doc.writexml)

    def read_contents(self):
        """Read file's contents and keep them in memory
        """
        self.__contents = self.read()

    def forget_contents(self):
        """Clear this object's contents memory
        """
        self.__contents = None

    def get_contents(self):
        """Get this object's contents memory
        """
        return self.__contents

    def read(self):
        """Read file's contents into a string

        :return: (string) contents read
        """
        with open(self.path, 'r', encoding='utf-8') as file:
            return file.read()

    def __write(self, writer):
        if os.path.exists(self.path):
            CheckedCall(f'attrib -R {self.path}')
        with open(self.path, 'w', encoding='utf-8') as file:
            writer(file)



"""
Python package
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
import os
from xml.dom import minidom
from pykrete.io.file.rtf import Rtf
from pykrete.io.file import File
from pykrete.tools import Externals


class NugetPackage:
    """NuGet package
    """
    __release_notes_element = 'releaseNotes'

    @property
    def package_file(self):
        """
        :return: Path to the package file, if packed by this instance
        """
        return self._package

    def __init__(self, nuspec_path, version, source=None, nuget_config=None):
        """Initialize this intance

        :param nuspec_path: Nuspec file path
        :param version: package version
        :param source: (optional) package source
        :param nuget_config: (optional) NuGet configuration file path
        """
        self._nuspec_path = nuspec_path
        self._version = version
        self._source = source
        self._nuget = Externals().nuget(nuget_config)
        self._package = None

    def update_release_notes(self, release_notes_rtf_path):
        """Update release notes in nuspec from the specified RTF file

        :param release_notes_rtf_path: RTF file path
        """
        nuspec_doc = minidom.parse(self._nuspec_path)
        rtf_text = Rtf(release_notes_rtf_path).text.strip()
        self._replace_release_notes(nuspec_doc, rtf_text)
        File(self._nuspec_path).write_doc(nuspec_doc)

    def pack(self, is_rc):
        """Creates a NuGet package

        :param is_rc: a flag indicating this is a release-candidate package
        """
        version_name = f'{self._version.number_str_without_build}' \
                       f'{"-rc" if is_rc else "."}{self._version.build}'
        output = self._nuget.run(
            ['pack', os.path.dirname(self._nuspec_path), '-Version', version_name,
             '-properties', f'Configuration={"Debug" if is_rc else "Release"}'],
            f'Failed to pack {self._nuspec_path} [{version_name}]')
        self._package = re.findall(r'Successfully created package \'([^\']*)\'', output)[0] \
            .strip()

    def push(self):
        """Pushes the package file packed by this instance
        """
        params = ['push', self._package] + (['-Source', self._source] if self._source else [])
        self._nuget.run(params, 'Failed to push ' + self._package)

    def _replace_release_notes(self, doc, text):
        release_notes = doc.getElementsByTagName(self.__release_notes_element)[0]
        cdata = doc.createCDATASection(text)
        release_notes.replaceChild(cdata, release_notes.firstChild)



"""
Pykrete repo.Git.git_internals.GitUrl tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

import unittest
from pykrete.repo.git_internals.git_urls import GitUrls
from .git_repo_testbed import GitRepoRemoteTestbed


class PythonGitUrlsTestCase(unittest.TestCase):
    """E2E tests for pykrete's repo module's git_internals.GitUrl class
    """

    def test_no_configured_ssh_urls(self):
        """Verifies no ssh urls returned when none were configured
        """
        target = GitUrls(GitRepoRemoteTestbed(['http://google.com/my.git',
                                               'https://example.org/project/his.git']))
        self.assertFalse(target.configured_ssh_urls, 'got ssh urls even though there were none')

    def test_some_configured_ssh_urls(self):
        """Verifies ssh urls returned when configured
        """
        expected_url = 'ssh://git@google.com:my.git'
        target = GitUrls(GitRepoRemoteTestbed(['http://google.com/my.git', expected_url]))
        self.assertEqual([expected_url], target.configured_ssh_urls, 'wrong/no URL returned')

    def test_configure_ssh_url_and_save_original_http(self):
        """Verifies correct replacement of http URL with SSH URL
        """
        self._test_configure_ssh_url_and_save_original(
            'http://google.com/project/my.git', '22',
            'ssh://git@google.com/project/my.git')

    def test_configure_ssh_url_and_save_original_https(self):
        """Verifies correct replacement of https URL with SSH URL
        """
        self._test_configure_ssh_url_and_save_original(
            'https://example.org/project/his.git', '22',
            'ssh://git@example.org/project/his.git')

    def test_configure_ssh_url_and_save_original_https_with_token(self):
        """Verifies correct replacement of tokenized https URL with SSH URL
        """
        self._test_configure_ssh_url_and_save_original(
            'https://gitlab-ci-token:[MASKED]'
            '@gitlab.devtools.intel.com/ait-projects/testing-grounds.git', '1234',
            'ssh://git@gitlab.devtools.intel.com:1234/ait-projects/testing-grounds.git')

    def test_configure_ssh_url_and_save_original_with_no_urls(self):
        """Verifies correct replacement of http URL with SSH URL
        """
        target = GitUrls(GitRepoRemoteTestbed([]))
        with self.assertRaises(IOError):
            target.configure_ssh_url_and_save_original('22')

    def _test_configure_ssh_url_and_save_original(self, original_url, port, expected_url):
        """Verifies correct replacement of http* URL with SSH URL
        """
        remote = GitRepoRemoteTestbed([original_url])
        target = GitUrls(remote)
        self.assertEqual([], target.configured_ssh_urls, 'ssh url returned before being configured')
        set_urls = target.configure_ssh_url_and_save_original(port)
        self.assertEqual((original_url, expected_url), set_urls, 'wrong history returned')
        self.assertEqual([(expected_url, original_url)], remote.urls_set, 'wrong url set')
        self.assertEqual([expected_url], target.configured_ssh_urls, 'wrong ssh url returned')


if __name__ == '__main__':
    unittest.main()



"""
Command-line tool
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from pykrete.patterns.common import existing_file_environment_cache
from pykrete.calls import CheckedCall


class CalledTool:
    """Manages a CLI tool whose path is detectable and cached in an environment variable
    """
    @property
    def path(self):
        """
        :return: Tool path
        """
        return self._path

    def __init__(self, tool_path_env_var, tool_path_detector, *args):
        """Initializes this instance

        :param tool_path_env_var: Environment variable caching this tool's path
        :param tool_path_detector: Method with no parameters which returns the tool's path
        :param args: (optional) parameters to pass in any call to the tool
        """
        self._path = existing_file_environment_cache(tool_path_env_var, tool_path_detector)
        self._base_call = [self._path] + list(args)

    def run(self, params, error):
        """Runs the tool with parameters, checking for success

        :param params: (str/list) call parameters
        :param error: (str) error message in case of failure
        :return: (str) tool's standard output
        """
        if isinstance(params, list):
            call = CheckedCall(self._base_call + params)
        else:
            base = " ".join(f'"{part}"' if ' ' in part else part for part in self._base_call)
            call = CheckedCall(f'{base} {str(params)}')
        call.assert_success(error)
        return call.stdout



"""
Visitor pattern base class
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""


class Target:
    """Visitor target base class"""

    def accept(self, visitor):
        """Accept a visitor

        :param visitor: visitor
        """
        return visitor.visit(self)



"""
.NET build manager
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import shutil
import logging
from pykrete.versioning import Version
from pykrete.source.dot_net import Solution, ProjectType
from pykrete.packages import restore_nuget_packages
from pykrete.args import environ, BuildMode
from pykrete.tools import VisualStudio
from .build_manager import BuildManager


class DotNetBuildManager(BuildManager):
    """Manages building a python package"""

    _logger = logging.getLogger(__name__)

    @property
    def mode(self):
        """
        :return: (BuildMode) the build mode used, or None if not built yet
        """
        return self._mode

    def __init__(self, version: Version, release_notes=None):
        """Initialize this instance

        :param version: Build version
        :param release_notes: (dict) release notes per nuspec (use * for default)
        """
        super().__init__(version)
        self._solution = Solution(self._project)
        self._release_notes = release_notes if release_notes else {}
        self._nuget_config = environ('NUGET_CONFIG')
        self._nuget_source = environ('NUGET_SOURCE')
        self._nugets = []
        self._mode = None

    def _clean_distribution(self):
        """Removes previous distribution files"""
        self._logger.debug('Deleting previous distribution')
        shutil.rmtree('binaries', ignore_errors=True)

    def _set_package_version(self):
        """Sets the package version in the project's version file"""
        print(f'Updating {self._project} projects\' version to {self._version}:')
        for project in self._solution.projects:
            project.assembly_info.version = self._version
            print(f' - updated {project}')

    def _build(self):
        """Performs the actual build"""
        self._logger.debug('Building %s', self._project)
        restore_nuget_packages(self._project, self._nuget_config)
        self._run_msbuild()
        self._pack_nugets()
        self._extract_artifacts()

    def _run_msbuild(self):
        self._mode = BuildMode.RELEASE if self._is_release else BuildMode.DEBUG
        is_multi_threaded = environ('MSBUILD_MULTITHREADED') is not None
        VisualStudio().build.run(
            [f'{self._project}', '/t:Clean,ReBuild', f'/p:Configuration={self._mode.name.title()}']
            + (['/m'] if is_multi_threaded else []),
            'failed to build ' + self._project)

    def _pack_nugets(self):
        default_notes = self._release_notes.get('*', None)
        for project in self._solution.projects:
            nuget = project.create_nuget_package_manager(self._nuget_source, self._nuget_config)
            if nuget:
                project.set_default_mode(BuildMode.RELEASE if self._is_release else BuildMode.DEBUG)
                notes = self._release_notes.get(project.name, default_notes)
                if notes:
                    nuget.update_release_notes(notes)
                nuget.pack(not self._is_release)
                self._nugets.append(nuget)

    def _extract_artifacts(self):
        mode = BuildMode.RELEASE if self._is_release else BuildMode.DEBUG
        relevant_projects = [project for project in self._solution.projects
                             if project.type != ProjectType.LIBRARY]
        for project in relevant_projects:
            project.extract_artifacts('binaries', mode)

    def _upload(self):
        """Uploads the package
        """
        self._logger.debug('Uploading %s', self._project)
        for nuget in self._nugets:
            nuget.push()



"""
.Net source solution
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import os
import re
from pykrete.args import exiting_file
from pykrete.io.file import File
from .project import Project


class Solution:
    """Handle a source-code solution

        Properties:
        path (string): the solution file's path
        projects ([SourceProject]): a vector of projects in the solution
    """

    @property
    def path(self):
        """
        :return: (string) the solution file's path
        """
        return self._path

    @property
    def projects(self):
        """
        :return: [Project] a list of projects in the solution
        """
        return self._projects

    def __init__(self, path):
        """Initializes this instance

        :param path: solution's path
        """
        self._path = exiting_file(path)
        self._projects = self.__locate_projects()

    def __locate_projects(self):
        """Extracts project paths from the relevant line in the solution file"""
        return [Project(os.path.join(os.path.dirname(self.path), path)) for
                path in re.findall(r'Project.*?FAE04EC0.*?, "([^"]+?)"', File(self.path).read())]



"""
Build tools
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from .python_build_manager import PythonBuildManager
from .dot_net_build_manager import DotNetBuildManager

__all__ = ['PythonBuildManager', 'DotNetBuildManager']



"""
INNO setup management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
from pykrete.io.file import File, ReReplacement, TextReplacement
from pykrete.calls.checked_call import CheckedCall


class InnoPackage:
    """Handles Inno setup packages
    """

    _inno_path = 'C:\\Program Files (x86)\\Inno Setup 6\\ISCC.exe'

    def __init__(self, mode, version):
        """Initializes this instance to manage the specified solution's installer

        :param mode: Package mode: debug / release
        :param version: Package version
        """
        self._mode = mode
        self._version = version

    def make_installer(self, script_path, temp_path):
        """Makes the installer exe using INNO tool

        :param script_path: INNO script path
        :param temp_path: temp folder path (holds MuDI Folder)
        :return: installer's path
        """
        File(script_path).update(lambda x: True, *self.__inno_script_replacements(temp_path))
        cmd = CheckedCall([self._inno_path, script_path,
                           f'/DConfig={self._mode.name.title()}'])
        cmd.assert_success(f'failed to make installer from {script_path}')
        setups = re.findall(r'Resulting Setup program filename is:[\s\S]*?(.*\.exe)', cmd.stdout)
        return setups[0].strip()

    def __inno_script_replacements(self, temp_path):
        return [
            ReReplacement(r'#define MyAppVersion ".*?"',
                          f'#define MyAppVersion "{self._version.number_str}"'),
            ReReplacement(r'#define MyVersionTitle ".*?"',
                          f'#define MyVersionTitle "{self._version.number_str_without_build}"'),
            TextReplacement('{#MainFolder}MuDiInstaller\\Temp\\', temp_path)
        ]



"""
GITLAB Upstream artifact downloader
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from zipfile import ZipFile
from .gitlab_str import project_str, job_str, project_pipeline_str


class GitlabUpstreamArtifacts:
    """Manage upstream artifacts"""

    def __init__(self, server, upstream_env):
        """Initialize this instance for a specific job's artifacts

        :param server: GitLab server
        :param upstream_env: upstream environment
        """
        self._logger = logging.getLogger(__name__)
        self._server = server
        self._upstream_env = upstream_env

    def download(self, target_path=None):
        """Download artifacts from last successful build job in the upstream pipeline

        :param target_path: Target path for downloads
        """
        job = self._get_upstream_build_job()
        self._extract_job_artifacts(job, target_path)

    def _get_upstream_build_job(self):
        project = self._get_upstream_project()
        pipeline = self._get_pipeline_by_id(project, self._upstream_env.pipeline_id)
        jobs = self._get_jobs_via_pipeline(
            pipeline, project,
            lambda x: x.attributes['name'].startswith(self._upstream_env.build_job_prefix))
        return jobs[-1]

    def _get_upstream_project(self):
        self._logger.debug('Locating project ID %s on GitLab', self._upstream_env.project_id)
        project = self._server.projects.get(self._upstream_env.project_id)
        self._logger.debug('Got GitLab project %s', project_str(project))
        return project

    def _get_pipeline_by_id(self, project, pipeline_id):
        self._logger.debug('Getting pipeline ID %s from GitLab project %s',
                           pipeline_id, project_str(project))
        pipeline = project.pipelines.get(pipeline_id)
        self._logger.debug('Got GitLab %s', project_pipeline_str(project, pipeline))
        return pipeline

    def _get_jobs_via_pipeline(self, pipeline, project, job_filter):
        pipeline_msg = project_pipeline_str(project, pipeline)
        self._logger.debug('Getting jobs from %s', pipeline_msg)
        jobs = [project.jobs.get(pipeline_job.id)
                for pipeline_job in pipeline.jobs.list() if job_filter(pipeline_job)]
        if not jobs:
            raise IOError('No matching jobs found in ' + pipeline_msg)
        self._logger.debug('Got GitLab jobs: [%s] from %s',
                           ",".join([job_str(job) for job in jobs]), pipeline_msg)
        return jobs

    def _extract_job_artifacts(self, job, target_path):
        zip_path = self._download_artifacts_zip(job)
        self._unzip(zip_path, target_path)

    def _download_artifacts_zip(self, job):
        zip_path = 'artifacts.zip'
        self._logger.debug('Downloading artifacts from job into %s', zip_path)
        with open(zip_path, 'wb') as file:
            job.artifacts(streamed=True, action=file.write)
        return zip_path

    def _unzip(self, zip_path, target_path):
        unzip_path = target_path if target_path else 'artifacts'
        self._logger.debug('Unzipping artifacts into %s', unzip_path)
        with ZipFile(zip_path, 'r') as zip_file:
            zip_file.extractall(unzip_path)



"""
Pykrete root
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

from .version import __version__

__all__ = ['__version__', 'args', 'builder', 'calls', 'ci', 'credentials', 'distrib', 'io',
           'logging', 'packages', 'patterns', 'repo', 'source', 'tools', 'versioning']



"""
Pattern classes
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from .common import assert_true, first_or_error, pykrete_root

__all__ = ['visitor', 'assert_true', 'first_or_error', 'pykrete_root']



"""
Python package installation
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from pykrete.tools import Externals


def restore_nuget_packages(sln_path, nuget_config=None, is_update=False):
    """Restores NuGet package
    """
    nuget = Externals().nuget(nuget_config)
    commands = ['restore'] + (['update'] if is_update else [])
    for command in commands:
        nuget.run([command, sln_path], 'failed to run nuget ' + command)



"""
version.py version information manager
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
import datetime
from os import path
from re import findall
from pykrete.io.file import write_lines
from .version import Version
from .revision_type import RevisionType
from .formatting import Formatting


def make_python_root_version_file_path(project):
    """ Generate the version.py file from the project's name

    :param project: project name
    :return: src/<project>/version.py
    """
    return path.join('src', project, 'version.py')


class VersionPyVersion(Version):
    """Handle version in python.py"""

    revisions = {
        '.': RevisionType.Release,
        'rc': RevisionType.RC,
        'b': RevisionType.Beta,
        'a': RevisionType.Alpha}

    def __init__(self, project, set_version=None):
        """Initialize this instance from the specified project's version.py file

        :param project: Python project (under 'src')
        :param set_version: Optional version to set instead of the one in the file (defaults to
         None, which means read the version from the file)
        """
        self._logger = logging.getLogger(__name__)
        self._project = project.title()
        self._version_py_path = make_python_root_version_file_path(project)
        if set_version:
            super().__init__(set_version)
        else:
            version_match = self.__read_version_match()
            version_parts = self.__parse_version_parts(version_match)
            super().__init__(version_parts)
        self._logger.debug('%s version.py version is %s', self._project, self)

    def apply(self):
        """Apply this version to the relevant environment
        """
        with open(self._version_py_path, 'w') as file:
            write_lines(file, self.__make_version_file_lines())
        self._logger.debug('Set %s version.py version to %s', self._project, self)

    def __read_version_match(self):
        try:
            with open(self._version_py_path, 'r') as file:
                separator_pattern = '|'.join(VersionPyVersion.revisions.keys()).replace('.', r'\.')
                return findall(f"__version__ = '(\\d+)\\.(\\d+)({separator_pattern})(\\d+)'",
                               file.read())[0]
        except IndexError:
            raise IOError("No version specification found in file")

    def __make_version_file_lines(self):
        """Makes the lines of version.py

        :return: The version file's lines
        """
        return [
            '"""',
            f'{self._project} module version',
            'AUTO-GENERATED BY BUILD SCRIPT',
            f'(C) {datetime.date.today().year}',
            '"""',
            '',
            f'__version__ = \'{Formatting(self).for_python}\'',
            ''
        ]

    @staticmethod
    def __parse_version_parts(version_parts):
        if not version_parts or len(version_parts) != 4:
            raise IOError('Incompatible version')
        revision_marker = version_parts[2]
        revision_type = VersionPyVersion.revisions[revision_marker]
        return (version_parts[0], version_parts[1], revision_type.value, version_parts[3],
                revision_type)



"""
.Net source project
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import os
import re
import logging
from pykrete.args import exiting_file
from pykrete.io.file import File, ReReplacement
from pykrete.patterns import first_or_error
from pykrete.packages import NugetPackage
from pykrete.calls import CheckedCall
from pykrete.patterns import assert_true
from .project_type import ProjectType
from .assembly_info import AssemblyInfo


class Project:
    """Handle source-code projects in a solution
    """

    __test_project_type_guid = '3AC096D0-A1C2-E12C-1390-A8335801FDAB'
    _logger = logging.getLogger(__name__)

    @property
    def path(self):
        """
        :return: (string) the project's path
        """
        return self._path

    @property
    def directory(self):
        """
        :return: (string) the project's directory
        """
        return os.path.dirname(self._path)

    @property
    def name(self):
        """
        :return: (string) the project's name
        """
        return self._name

    @property
    def type(self):
        """
        :return: (ProjectType) the project's type
        """
        return self._type

    @property
    def assembly_info(self):
        """
        :return: (AssemblyInfo) the project's assembly info
        """
        return self._assembly_info

    def __init__(self, path):
        """Initializes this instance

        :param path: project's path
        """
        self._path = exiting_file(path)
        self.__file = File(path)
        self.__file.read_contents()
        self._name = self.__locate_name()
        self._type = self.__locate_type()
        self._output = self.__locate_output()
        self._assembly_info = self.__locate_assembly_info()
        self._nuspec = self.__locate_nuspec()
        self.__file.forget_contents()

    def create_nuget_package_manager(self, source=None, nuget_config=None):
        """Creates a nuget package manager for this project, if it has a nuspec in it

        :param source: NuGet source (optional)
        :param nuget_config: NuGet config path (optional)
        :return: (NugetPackage) nuget package manager, or None if no nuspec is present
        """
        return NugetPackage(self._nuspec, self._assembly_info.version, source, nuget_config) \
            if self._nuspec else None

    def set_default_mode(self, mode):
        """Sets the default mode for this project

        :param mode: (BuildMode) default mode
        """
        config_element_open = '<Configuration Condition=" \'$(Configuration)\' == \'\' ">'
        config_element_open_re = re.sub(r'(\$|\(|\))', r'\\\1', config_element_open)
        config_element_close = '</Configuration>'
        assert_true(
            File(self._path).update(
                lambda x: config_element_open in x,
                ReReplacement(
                    f'{config_element_open_re}.*?{config_element_close}',
                    f'{config_element_open}{mode.name.title()}{config_element_close}',
                )), 'Default configuration not found in ' + self._path)

    def __str__(self):
        """Returns a string representation of this object

        :return: a string representation of this object
        """
        return f'{self.name} [{self.directory}]'

    def extract_artifacts(self, destination, mode):
        """Extracts artifacts from the project's output

        :param destination: path into which artifacts are extracted
        :param mode: build mode of the artifacts
        """
        action_str = f'Copying artifacts from {self._name} [{self._assembly_info.include}]'
        self._logger.debug(action_str)
        CheckedCall(
            ['xcopy',
             f'{os.path.join(self._output[mode.name.title()], self._assembly_info.include)}',
             f'{os.path.join(destination, self._type.name, self._name, "")}',
             '/E', '/I', '/Y', '/F']) \
            .assert_success(f'Failed while {action_str}')

    def __locate_name(self):
        return first_or_error(re.findall(r'<AssemblyName>(.*?)</', self.__file.get_contents()),
                              'AssemblyName not found in ' + self.path)

    def __locate_type(self):
        output_type = re.findall(r'<OutputType>(.+?)</OutputType>', self.__file.get_contents())[0]
        if output_type == "Library":
            if self.__test_project_type_guid in self.__file.get_contents():
                return ProjectType.UNIT_TEST
            if self.name.endswith("Tests") or self.name.endswith("Test"):
                return ProjectType.FLOW_TEST
            return ProjectType.LIBRARY
        if output_type.endswith("Exe"):
            return ProjectType.EXECUTABLE
        return ProjectType.UNKNOWN

    def __locate_output(self):
        mode_and_out_path = re.findall(
            r'<PropertyGroup Condition=" \'\$\(C.+?\)\|.+?\)\'.+_?\'(.+?)\|[\s\S]+?'
            r'<OutputPath>(.+?)<', self.__file.get_contents())
        return {mode: os.path.join(self.directory, out_path)
                for (mode, out_path) in mode_and_out_path}

    def __locate_assembly_info(self):
        return AssemblyInfo(
            os.path.join(
                self.directory,
                first_or_error(
                    re.findall(r'<Compile Include="(.*AssemblyInfo\.cs)"',
                               self.__file.get_contents()),
                    'AssemblyInfo not found in ' + self.path)))

    def __locate_nuspec(self):
        nuspecs = [f for f in os.listdir(self.directory) if f.endswith('.nuspec')]
        return os.path.join(self.directory, nuspecs[0]) if nuspecs else None



"""
Handle SSH private key
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from os import remove, getcwd, path
from uuid import uuid4
from pykrete.args import CiIo
from pykrete.patterns.visitor import Visitor
from .state import State


class CiSshPrivateKey(Visitor):
    """Handle SSH private key specifid in the CI environment"""

    @property
    def is_enabled(self):
        """
        :return: True if the key is enabled, False otherwise
        """
        return self._state == State.Enabled

    @property
    def ssh_port(self):
        """
        :return: the CI-configured SSH port
        """
        if not self._ssh_port:
            self._ssh_port = self.__ci_io.read_env('ssh port', '22')
        return self._ssh_port

    def __init__(self, ci_io=None, key_file=None):
        """Initialize this instance

        :param ci_io: CI environment reader override
        """
        self._logger = logging.getLogger(__name__)
        self.__ci_io = ci_io if ci_io else CiIo()
        self._state = State.NotInitialized
        self._key_file = key_file
        self._ssh_port = None

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.disable()

    def visit(self, target):
        """Visit this instance

        :param target: visited target
        """
        self._enable()
        target.set_ssh(self._key_file, self.ssh_port)

    def disable(self):
        """Disables the private key

        :exception: PermissionError - key already disabled
        """
        if self._must_not_be_disabled_but_is(State.NotInitialized):
            return
        self._logger.debug('Disabling SSH from key-file=%s', self._key_file)
        remove(self._key_file)
        self._state = State.Disabled

    def _enable(self):
        if self._must_not_be_disabled_but_is(State.Enabled):
            return
        self._make_key_file()
        self._state = State.Enabled

    def _make_key_file(self):
        if not self._key_file:
            self._key_file = path.join(getcwd(), str(uuid4()))
        key = self.__ci_io.read_env('deploy key')
        self._logger.debug('Enabling SSH with key-file=%s', self._key_file)
        with open(self._key_file, 'w') as key_file:
            key_file.write(key)

    def _must_not_be_disabled_but_is(self, other_state):
        if self._state == State.Disabled:
            self._logger.error('Trying to operate disabled SSH key-file')
            raise PermissionError('Credentials were disabled')
        return self._state == other_state



"""
Strip non-textual RTF contents
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
Sources:
    https://stackoverflow.com/a/93029/11750328
    http://stackoverflow.com/a/188877
"""
import re


class RtfStripper:
    """Strips non-text data from an RTF string

    Properties:
    rtf (string): original string
    text (string): stripped string
    """
    __pattern = re.compile(
        r"\\([a-z]{1,32})(-?\d{1,10})?[ ]?|\\'([0-9a-f]{2})|\\([^a-z])|([{}])|[\r\n]+|(.)", re.I)
    # control words which specify a "destination".
    __destinations = frozenset((
        'aftncn', 'aftnsep', 'aftnsepc', 'annotation', 'atnauthor', 'atndate', 'atnicn', 'atnid',
        'atnparent', 'atnref', 'atntime', 'atrfend', 'atrfstart', 'author', 'background',
        'bkmkend', 'bkmkstart', 'blipuid', 'buptim', 'category', 'colorschememapping',
        'colortbl', 'comment', 'company', 'creatim', 'datafield', 'datastore', 'defchp', 'defpap',
        'do', 'doccomm', 'docvar', 'dptxbxtext', 'ebcend', 'ebcstart', 'factoidname', 'falt',
        'fchars', 'ffdeftext', 'ffentrymcr', 'ffexitmcr', 'ffformat', 'ffhelptext', 'ffl',
        'ffname', 'ffstattext', 'field', 'file', 'filetbl', 'fldinst', 'fldrslt', 'fldtype',
        'fname', 'fontemb', 'fontfile', 'fonttbl', 'footer', 'footerf', 'footerl', 'footerr',
        'footnote', 'formfield', 'ftncn', 'ftnsep', 'ftnsepc', 'g', 'generator', 'gridtbl',
        'header', 'headerf', 'headerl', 'headerr', 'hl', 'hlfr', 'hlinkbase', 'hlloc', 'hlsrc',
        'hsv', 'htmltag', 'info', 'keycode', 'keywords', 'latentstyles', 'lchars', 'levelnumbers',
        'leveltext', 'lfolevel', 'linkval', 'list', 'listlevel', 'listname', 'listoverride',
        'listoverridetable', 'listpicture', 'liststylename', 'listtable', 'listtext',
        'lsdlockedexcept', 'macc', 'maccPr', 'mailmerge', 'maln', 'malnScr', 'manager', 'margPr',
        'mbar', 'mbarPr', 'mbaseJc', 'mbegChr', 'mborderBox', 'mborderBoxPr', 'mbox', 'mboxPr',
        'mchr', 'mcount', 'mctrlPr', 'md', 'mdeg', 'mdegHide', 'mden', 'mdiff', 'mdPr', 'me',
        'mendChr', 'meqArr', 'meqArrPr', 'mf', 'mfName', 'mfPr', 'mfunc', 'mfuncPr', 'mgroupChr',
        'mgroupChrPr', 'mgrow', 'mhideBot', 'mhideLeft', 'mhideRight', 'mhideTop', 'mhtmltag',
        'mlim', 'mlimloc', 'mlimlow', 'mlimlowPr', 'mlimupp', 'mlimuppPr', 'mm', 'mmaddfieldname',
        'mmath', 'mmathPict', 'mmathPr', 'mmaxdist', 'mmc', 'mmcJc', 'mmconnectstr',
        'mmconnectstrdata', 'mmcPr', 'mmcs', 'mmdatasource', 'mmheadersource', 'mmmailsubject',
        'mmodso', 'mmodsofilter', 'mmodsofldmpdata', 'mmodsomappedname', 'mmodsoname',
        'mmodsorecipdata', 'mmodsosort', 'mmodsosrc', 'mmodsotable', 'mmodsoudl',
        'mmodsoudldata', 'mmodsouniquetag', 'mmPr', 'mmquery', 'mmr', 'mnary', 'mnaryPr',
        'mnoBreak', 'mnum', 'mobjDist', 'moMath', 'moMathPara', 'moMathParaPr', 'mopEmu',
        'mphant', 'mphantPr', 'mplcHide', 'mpos', 'mr', 'mrad', 'mradPr', 'mrPr', 'msepChr',
        'mshow', 'mshp', 'msPre', 'msPrePr', 'msSub', 'msSubPr', 'msSubSup', 'msSubSupPr', 'msSup',
        'msSupPr', 'mstrikeBLTR', 'mstrikeH', 'mstrikeTLBR', 'mstrikeV', 'msub', 'msubHide',
        'msup', 'msupHide', 'mtransp', 'mtype', 'mvertJc', 'mvfmf', 'mvfml', 'mvtof', 'mvtol',
        'mzeroAsc', 'mzeroDesc', 'mzeroWid', 'nesttableprops', 'nextfile', 'nonesttables',
        'objalias', 'objclass', 'objdata', 'object', 'objname', 'objsect', 'objtime', 'oldcprops',
        'oldpprops', 'oldsprops', 'oldtprops', 'oleclsid', 'operator', 'panose', 'password',
        'passwordhash', 'pgp', 'pgptbl', 'picprop', 'pict', 'pn', 'pnseclvl', 'pntext', 'pntxta',
        'pntxtb', 'printim', 'private', 'propname', 'protend', 'protstart', 'protusertbl', 'pxe',
        'result', 'revtbl', 'revtim', 'rsidtbl', 'rxe', 'shp', 'shpgrp', 'shpinst',
        'shppict', 'shprslt', 'shptxt', 'sn', 'sp', 'staticval', 'stylesheet', 'subject', 'sv',
        'svb', 'tc', 'template', 'themedata', 'title', 'txe', 'ud', 'upr', 'userprops',
        'wgrffmtfilter', 'windowcaption', 'writereservation', 'writereservhash', 'xe', 'xform',
        'xmlattrname', 'xmlattrvalue', 'xmlclose', 'xmlname', 'xmlnstbl',
        'xmlopen',
    ))
    # Translation of some special characters.
    __specialchars = {
        'par': '\n',
        'sect': '\n\n',
        'page': '\n\n',
        'line': '\n',
        'tab': '\t',
        'emdash': '\u2014',
        'endash': '\u2013',
        'emspace': '\u2003',
        'enspace': '\u2002',
        'qmspace': '\u2005',
        'bullet': '\u2022',
        'lquote': '\u2018',
        'rquote': '\u2019',
        'ldblquote': '\201C',
        'rdblquote': '\u201D',
    }

    @property
    def text(self):
        """
        :return: Text stripped from the RTF file
        """
        return self._text

    def __init__(self, rtf):
        """Initialize this class by performing the stripping and saving the result

        :param rtf: source string
        """
        self.rtf = rtf
        self.__stack = []
        self.__is_ignorable_group = False
        self.__chars_to_skip_after_unicode = 1
        self.__chars_left_to_skip = 0
        self.__out = []  # Output buffer.
        self.__strip()
        self._text = ''.join(self.__out)

    def __strip(self):
        for match in self.__pattern.finditer(self.rtf):
            word, arg, hexa, char, brace, tchar = match.groups()
            if brace:
                self.__handle_brace(brace)
            elif char:  # \x (not a letter)
                self.__handle_char(char)
            elif word:  # \foo
                self.__handle_word(arg, word)
            elif hexa:  # \'xx
                self.__handle_hexa(hexa)
            elif tchar:
                self.__handle_tchar(tchar)

    def __handle_tchar(self, tchar):
        if self.__chars_left_to_skip > 0:
            self.__chars_left_to_skip -= 1
        elif not self.__is_ignorable_group:
            self.__out.append(tchar)

    def __handle_hexa(self, hexa):
        if self.__chars_left_to_skip > 0:
            self.__chars_left_to_skip -= 1
        elif not self.__is_ignorable_group:
            char = int(hexa, 16)
            if char > 127:
                self.__out.append(chr(char))  # NOQA
            else:
                self.__out.append(chr(char))

    def __handle_word(self, arg, word):
        self.__chars_left_to_skip = 0
        if word in self.__destinations:
            self.__is_ignorable_group = True
        elif self.__is_ignorable_group:
            pass
        elif word in self.__specialchars:
            self.__out.append(self.__specialchars[word])
        elif word == 'uc':
            self.__chars_to_skip_after_unicode = int(arg)
        elif word == 'u':
            self.__handle_word_u(arg)

    def __handle_word_u(self, arg):
        char = int(arg)
        if char < 0:
            char += 0x10000
        if char > 127:
            self.__out.append(chr(char))  # NOQA
        else:
            self.__out.append(chr(char))
        self.__chars_left_to_skip = self.__chars_to_skip_after_unicode

    def __handle_char(self, char):
        self.__chars_left_to_skip = 0
        if char == '~':
            if not self.__is_ignorable_group:
                self.__out.append('\xA0')
        elif char in '{}\\':
            if not self.__is_ignorable_group:
                self.__out.append(char)
        elif char == '*':
            self.__is_ignorable_group = True

    def __handle_brace(self, brace):
        self.__chars_left_to_skip = 0
        if brace == '{':
            self.__stack.append((self.__chars_to_skip_after_unicode, self.__is_ignorable_group))
        elif brace == '}':
            self.__chars_to_skip_after_unicode, self.__is_ignorable_group = self.__stack.pop()



"""
Visual studio tools
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
import os
from pykrete.calls.checked_call import CheckedCall
from pykrete.args import build_version
from .called_tool import CalledTool


class VisualStudio:
    """Microsoft Visual Studio tools wrapper
    """

    _vswhere_exe = '"%ProgramFiles(x86)%\\Microsoft Visual Studio\\Installer\\vswhere.exe"'
    _vswhere_params = ' -latest -requires Microsoft.Component.MSBuild'
    _msbuild_environment_variable = "VSTOOLS_LATEST_MSBUILD"
    _mstest_environment_variable = "VSTOOLS_LATEST_MSTEST"

    @property
    def path(self):
        """
        :return: (str) Visual Studio installation path
        """
        return self._path

    @property
    def version(self):
        """
        :return: (match) Visual Studio version used
        """
        return self._version

    @property
    def build(self):
        """
        :return: (CalledTool) MS Build tool wrapper
        """
        if self._ms_build is None:
            self._ms_build = CalledTool(self._msbuild_environment_variable, self._detect_msbuild)
        return self._ms_build

    @property
    def test(self):
        """
        :return: (CalledTool) MS Test tool wrapper
        """
        if self._ms_test is None:
            self._ms_test = CalledTool(self._mstest_environment_variable, self._detect_mstest)
        return self._ms_test

    def __init__(self):
        """Initializes this instance for the latest VS
        """
        vswhere = CheckedCall(self._vswhere_exe + self._vswhere_params)
        if not vswhere.success:
            raise SystemError('Failed to location VS installation: ' + vswhere.stderr)

        parts = dict(re.findall(r'installation(Path|Version): (.*)', vswhere.stdout))

        self._path = parts["Path"].strip()
        self._version = build_version(parts["Version"].strip()).group(1)
        self._ms_build = None
        self._ms_test = None

    def __str__(self):
        return f'Path: {self.path}; Version: {self.version}'

    def _detect_msbuild(self):
        """Gets the detected msbuild tool path

        :return: path to msbuild tool
        """
        return [path for path in
                [os.path.join(self.path, 'MSBuild', option, 'Bin', 'msbuild.exe') for option in
                 ['current', f'{self.version}.0']]
                if os.path.isfile(path)][0]

    def _detect_mstest(self):
        """Gets the detected mstest tool path, cached by an environment variable

        :return: path to mstest tool
        """
        return f'{self.path}\\Common7\\IDE\\mstest.exe'



"""
Common module
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import os
import argparse
from pykrete.args import exiting_file


def assert_true(target, error):
    """Assert success of target

    :param target: target with bool success attribute
    :param error: error message in case of failure
    """
    if not target:
        raise AssertionError(error)


def first_or_error(vector, error):
    """Gets the first value in a vector, or throws an exception if the vector is empty

    :param vector: the target vector
    :param error: the error message
    :return: the first value in the vector
    """
    if vector:
        return vector[0]
    raise KeyError(error)


def existing_file_environment_cache(variable, path_maker):
    """Gets an existing file whose path is created once and kept in an environment variable

    :param variable: environment variable name
    :param path_maker: path maker lambda
    :return: path of existing file
    :exception: file doesn't exist
    """
    try:
        return exiting_file(os.environ[variable])
    except (KeyError, argparse.ArgumentTypeError):
        path = exiting_file(path_maker())
        os.environ[variable] = path
        return path


def pykrete_root():
    """
    :return: this package's installation root
    """
    package = 'pykrete'
    this_file = str(__file__)
    root = this_file[0:this_file.rfind(package)+len(package)]
    return root



"""
Python package installation
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
import subprocess


def install_python_packages(*args):
    """Installs the specified pip packages

    :param args: The packages to install
    """
    packages = list(args)
    logging.getLogger(__name__).debug('Installing packages %s', packages)
    subprocess.check_call(['python', '-m', 'pip', 'install'] + packages)



"""
Security credentials target
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from abc import abstractmethod

from pykrete.patterns.visitor import Target


class SshTarget(Target):
    """Visitor pattern - base class for credential targets
    """

    @abstractmethod
    def set_ssh(self, private_key_file_path, ssh_port):
        """Set the SSH credentials from the supplied file

        :param private_key_file_path: path to private key file
        :param ssh_port: SSH port
        """



"""
Pykrete credentials tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import os
import unittest
import logging
from uuid import uuid4
from pykrete.args import CiIo
from pykrete.credentials import CiSshPrivateKey, SshTarget
from .ci import PykreteCiEchoTestCase


class PykreteCredentialsUnitTests(PykreteCiEchoTestCase, SshTarget):
    """Test pykrete's credentials module"""

    _logger = logging.getLogger(__name__)
    _dummy_key = str(uuid4())
    _dummy_file = str(uuid4())
    _set_file = None

    def test_enable_disable_ci_ssh_private_key(self):
        """Verifies successful enabling and disabling of CI SSH private key credentials
        """
        ci_io = CiIo(self._echo, {'deploy key': self._dummy_key, 'ssh port': '1234'})
        with CiSshPrivateKey(ci_io, self._dummy_file) as target:
            self.accept(target)
            self.assertEqual(self._dummy_file + '1234', self._set_file,
                             'wrong file and port set on visit')
            self._assert_key_file()
        self.assertFalse(os.path.exists(self._dummy_file), 'key file still exists after exit')

    def _assert_key_file(self):
        with open(self._dummy_file, 'r') as file:
            key_in_file = file.read()
        self.assertEqual(self._dummy_key, key_in_file, 'wrong key written to file')

    def set_ssh(self, private_key_file_path, ssh_port):
        self._set_file = private_key_file_path + ssh_port


if __name__ == '__main__':
    unittest.main()



"""
Replacement base class
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from abc import abstractmethod


class ReplacementBase:
    """Replacement base class
    """
    _logger = logging.getLogger(__name__)

    def __init__(self, find, replacement):
        """Initialize this instance with the specified find and replacement

        :param find: (string) search string
        :param replacement: (string) replacement string
        """
        self._find = find
        self._replacement = replacement

    def replace(self, target):
        """Perform this replacement on the specified target

        :param target: (string) replacement target string
        """
        result = self._do_replace(target)
        self._logger.debug('[%s] :: before [%s] :: after [%s]', self, target, result)
        return result

    @abstractmethod
    def _do_replace(self, target):
        pass

    def __str__(self):
        return f'find: {self._find}; replace: {self._replacement}'



"""
GIT repository tag reading management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
import logging


class GitTagsReader:
    """Manage reading git tags"""

    @property
    def tags(self):
        """
        :return: all tags in the repo
        """
        tag_refs = self._get_tag_refs_by_date_ascending()
        return self.__name_message_tuples_from_tag_refs(tag_refs)

    def __init__(self, repo):
        """Initialize this instance

        :param repo: target repository
         subset (optional, defaults to getting all tags)
        """
        self._logger = logging.getLogger(__name__)
        self._repo = repo
        self._repo.remotes['origin'].fetch('--tags')

    def get_last_tag_ref_named(self, name, is_must=True):
        """Gets the last tag reference with the specified name

        :param name: tag name
        :param is_must: True to raise an error tag doesn't exist,
         False to just return None in that case.
        :return: tag reference
        :exception: KeyError - tag not found
        """
        tag_ref = self.__get_last_tag_ref_named(name, is_must)
        tag_tuple = self.name_message_tuple_from_tag_ref(tag_ref)
        self._logger.debug('Read tag from repo [%s]: %s', self._repo, tag_tuple)
        return tag_ref

    def get_tags_from(self, pattern, is_must):
        """Gets all tags from the current branch in the repo, from the tag whose name matches the
         supplied pattern.

        :param pattern: name pattern to match
        :param is_must: True to raise an error if no match is found, False to return all tags in
        that case (with the first tuple as None).
        :return: A list of tag info tuples (name, message)
        :exception: KeyError - tag not found
        """
        tag_refs = self.__get_tag_refs_from(pattern, is_must)
        return self.__name_message_tuples_from_tag_refs(tag_refs)

    @staticmethod
    def name_message_tuple_from_tag_ref(tag):
        """Convert a tag reference to a (name, message) tuple

        :param tag: tag reference
        :return: (name, message) tuple
        """
        return (tag.name, tag.tag.message if tag.tag else None) if tag else None

    def _get_tag_refs_by_date_ascending(self):
        tag_refs = list(self._repo.tags)
        tag_refs.sort(key=lambda x: x.tag.tagged_date if x and x.tag else 0)
        return tag_refs

    def __get_last_tag_ref_named(self, name, is_must):
        tag_refs = self._get_tag_refs_by_date_ascending()
        index = GitTagsReader.__find_last_tag_index(name.__eq__, tag_refs)
        return tag_refs[index] if \
            self.__is_found(index, is_must, f'No tag named {name} was found in repo') else \
            None

    def __get_tag_refs_from(self, pattern, is_must):
        tag_refs = self._get_tag_refs_by_date_ascending()
        from_index = GitTagsReader.__find_last_tag_index_by_name_pattern(pattern, tag_refs)
        return tag_refs[from_index:] if \
            self.__is_found(from_index, is_must, "No tag found to match pattern " + pattern) else \
            [None] + tag_refs

    def __name_message_tuples_from_tag_refs(self, tags):
        tag_tuples = [GitTagsReader.name_message_tuple_from_tag_ref(tag) for tag in tags]
        self._logger.debug('Read tags from repo [%s]: %s', self._repo, tag_tuples)
        return tag_tuples

    @staticmethod
    def __is_found(index, is_must, error):
        if index == -1:
            if is_must:
                raise KeyError(error)
            return False
        return True

    @staticmethod
    def __find_last_tag_index_by_name_pattern(pattern, tag_refs):
        compiled_pattern = re.compile(pattern)
        return GitTagsReader.__find_last_tag_index(compiled_pattern.match, tag_refs)

    @staticmethod
    def __find_last_tag_index(matcher, tag_refs):
        for i in range(len(tag_refs) - 1, -1, -1):
            if matcher(tag_refs[i].name):
                return i
        return -1



"""
Feature revision types
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from enum import Enum


class RevisionType(Enum):
    """Revision type enumeration"""
    PreAlpha = 0
    """Feature still in bring-up phase, mostly not testable"""

    Alpha = 1
    """Feature implementation in progress, testing enabled"""

    Beta = 2
    """Feature complete, fully testable"""

    RC = 3
    """Feature finalized, integration candidate"""

    Release = 4
    """Feature ready for production"""



"""
Packaged external tools
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import os
from pykrete.patterns import pykrete_root
from .called_tool import CalledTool


class Externals:
    """Wrappers for packaged external tools"""
    _nuget_environment_variable = "PYKRETE_NUGET"

    def nuget(self, config=None):
        """
        :param config: (optional) NuGet configuration file path
        :return: (CalledTool) NuGet tool wrapper
        """
        if self._nuget is None:
            if config is None:
                self._nuget = CalledTool(self._nuget_environment_variable, self._external_nuget)
            else:
                self._nuget = CalledTool(self._nuget_environment_variable, self._external_nuget,
                                         '-ConfigFile', config)
        return self._nuget

    def __init__(self):
        """Initializes this instance"""
        self._nuget = None

    def __str__(self):
        return f'Path: {self.pykrete_externals("")}'

    @staticmethod
    def pykrete_externals(filename):
        """
        :param filename: file under 'externals' folder
        :return: path to an externals file in this package's intallation folder
        """
        return os.path.join(pykrete_root(), 'externals', filename)

    def _external_nuget(self):
        nuget_exe = 'nuget.exe'
        return self.pykrete_externals(nuget_exe)



"""
GITLAB CI upstream management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from gitlab import Gitlab
from pykrete.args import CiIo
from .gitlab_upstream_env import GitLabUpstreamEnv
from .gitlab_upstream_atrifacts import GitlabUpstreamArtifacts


class GitLabUpstream:
    """GitLab CI upstream management"""

    @property
    def artifacts(self):
        """
        :return: Upstream artifacts
        """
        if not self._artifacts:
            self._artifacts = GitlabUpstreamArtifacts(
                server=self.__gitlab,
                upstream_env=self.__upstream_env)
        return self._artifacts

    def __init__(self, url, ci_io=None):
        """Initialize this instance

        :param url: gitlab URL
        :param ci_io: CI environment's IO manager (optional, defaults to pykrete.args.CiIo)
        """
        self._logger = logging.getLogger(__name__)
        self.__ci_io = ci_io if ci_io else CiIo()
        self.__upstream_env = GitLabUpstreamEnv()
        self.__gitlab = self._get_gitlab_connection_with_upstream_key_to_api_url(url)
        self._artifacts = None

    def _get_gitlab_connection_with_upstream_key_to_api_url(self, url):
        server = Gitlab(url, private_token=self.__upstream_env.key)
        self._logger.debug('Attempting connection to %s with the upstream token', url)
        server.auth()
        self._logger.debug('Connection to %s succeeded', url)
        return server



"""
GITLAB object stringify
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""


def project_str(project):
    """Stringify project
    :param project: GitLab project
    :return: Project name with namespace
    """
    return project.attributes['name_with_namespace']


def pipeline_str(pipeline):
    """Stringify pipeline
    :param pipeline: GitLab pipeline
    :return: pipeline reference with status
    """
    p_attr = pipeline.attributes
    return f'{p_attr["ref"]}[{p_attr["sha"][0:8]}/{p_attr["id"]} - {p_attr["status"]}]'


def project_pipeline_str(project, pipeline):
    """Stringify project pipeline
    :param project: GitLab project
    :param pipeline: GitLab pipeline
    :return: pipeline with project
    """
    return f'pipeline {pipeline_str(pipeline)} in GitLab project {project_str(project)}'


def job_str(job):
    """Stringify job
    :param job: GitLab job
    :return: job reference with commit info
    """
    j_attr = job.attributes
    commit_attr = 'commit'
    if commit_attr in j_attr:
        j_commit = j_attr[commit_attr]
        j_commit_str = f'{j_commit["title"]} by {j_commit["committer_name"]}'
    else:
        j_commit_str = 'no commit data'
    return f'{j_attr["ref"]}/{j_attr["name"]}/{j_attr["id"]}[{j_commit_str}]'



"""
GITLAB CI management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from urllib.parse import urlparse
from pykrete.args import CiIo
from .gitlab_internals.gitlab_upstream import GitLabUpstream


class GitLab:
    """GitLab CI management"""

    @property
    def upstream(self):
        """
        :return: upstream CI management
        """
        if not self._upstream:
            self._upstream = GitLabUpstream(self._get_gitlab_api_url(), self.__ci_io)
        return self._upstream

    def __init__(self, ci_io=None):
        """Initialize this instance

        :param ci_io: CI environment's IO manager (optional, defaults to pykrete.args.CiIo)
        """
        self._logger = logging.getLogger(__name__)
        self.__ci_io = ci_io if ci_io else CiIo()
        self._upstream = None

    def _get_gitlab_api_url(self):
        parsed_uri = urlparse(self.__ci_io.read_env('api url'))
        url = f'{parsed_uri.scheme}://{parsed_uri.hostname}/'
        return url



"""
CI information manager
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
from .environ import environ


class CiIo:
    """Handle CI environment IO
    """
    def __init__(self, ci_reader=environ, ci_spec=None):
        """Initializes this instance from CI environment

        :param ci_reader: (ci_variable, role)=>value function [see pykrete.args.environ behavior]
        :param ci_spec: dictionary of part to CI variable
         (parts are: 'major version', 'minor version', 'build version', 'branch name',
          'merge request title', 'job name', 'deploy key')
        """
        self._logger = logging.getLogger(__name__)
        self.__ci_reader = ci_reader
        self.__ci_spec = ci_spec if ci_spec else self.__get_default_ci_spec()

    def read_env(self, part, default=None):
        """Read a spec-part from the CI environment

        :param part: part name
        :param default: default value if not found,
         or None if there should be an exception if not found
        :return: Read value, or None if not found
        :exception: The part's variable is not defined in the environment and it should be
        """
        role = None if default is not None else f'CI\'s \'{part}\' value'
        var_name = self.__ci_spec[part]
        self._logger.debug('Reading CI environment %s [role: %s] from %s', part, role, var_name)
        value = self.__ci_reader(var_name, role)
        if default and not value:
            value_source = 'Used default'
            value = default
        else:
            value_source = 'Read'
        self._logger.debug('%s %s = %s', value_source, part, '[MASKED]' if 'key' in part else value)
        return value

    @staticmethod
    def __get_default_ci_spec():
        return {'major version': 'CI_VERSION_MAJOR',
                'minor version': 'CI_VERSION_MINOR',
                'build version': 'CI_PIPELINE_IID',
                'branch name': 'CI_COMMIT_REF_NAME',
                'merge request title': 'CI_MERGE_REQUEST_TITLE',
                'job name': 'CI_JOB_NAME',
                'deploy key': 'CI_DEPLOY_KEY',
                'ssh port': 'SSH_PORT',
                'upstream project ID': 'UPSTREAM_PROJECT_ID',
                'upstream pipeline ID': 'UPSTREAM_PIPELINE_ID',
                'upstream build job prefix': 'UPSTREAM_BUILD_JOB_PREFIX',
                'upstream key': 'UPSTREAM_TOKEN',
                'api url': 'CI_API_V4_URL'}



"""
Extract text from RTF
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
from pykrete.io.file import File
from .rtf_stripper import RtfStripper


class Rtf:
    """ Read an RTF file's text contenxt
    """
    __control_chars = ''.join(map(chr, list(range(0, 9)) + list(range(14, 18)) +
                                  list(range(127, 160))))
    __control_char_re = re.compile('[%s]' % re.escape(__control_chars))

    @property
    def text(self):
        """
        :return: Text from the RTF file
        """
        return self._text

    def __init__(self, path):
        """Initialize this instance

        :param path: RTF file's path
        """
        rtf = self.__read_rtf_without_control_chars(path)
        self._text = RtfStripper(rtf).text

    def __read_rtf_without_control_chars(self, path):
        rtf = File(path).read()
        clean_rtf = self.__control_char_re.sub('', rtf)
        return clean_rtf



"""
Security credentials management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from .ssh_target import SshTarget
from .ci_ssh_private_key import CiSshPrivateKey

__all__ = ['SshTarget', 'CiSshPrivateKey']



"""
.Net source project's assembly info
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
from pykrete.io.file import File, ReReplacement


class AssemblyInfo:
    """.Net project's assembly info management"""

    @property
    def include(self):
        """
        :return: included artifacts, or '*' if none are defined
        """
        return self._include

    @property
    def version(self):
        """
        :return: (Version) project version
        """
        return self._version

    @version.setter
    def version(self, value):
        """Set project version

        :param value: (Version) project version
        """
        self._version = value
        self.__apply_version()

    def __init__(self, path):
        """Initialize this instance

        :param path: path to assembly info file
        """
        self._path = path
        self._file = File(path)
        self._include = self.__extract_artifacts()
        self._version = None

    def __extract_artifacts(self):
        # doesn't seem to work for MuDi.Cli!
        include = re.findall(r'\[assembly: Artifact"\(.*\)"\]', self._file.read())
        return include if include else "*"

    def __apply_version(self):
        if not self._file.update(
                lambda x: 'assembly: Assembly' in x,
                ReReplacement(r'Version\(".*?"\)', f'Version("{self._version.number_str}")')):
            raise KeyError('No assembly information found in ' + self._path)



"""
Regex replacement class
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import re
from pykrete.io.file.replacement_base import ReplacementBase


class ReReplacement(ReplacementBase):
    """Holds regular-expression text replacement request
    """

    def _do_replace(self, target):
        return re.sub(self._find, self._replacement, target)



"""
Credentials state
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from enum import Enum


class State(Enum):
    """Credentials state"""

    NotInitialized = 0
    """Credentials not initialized"""

    Enabled = 1
    """Credentials enabled"""

    Disabled = 2
    """Credentials disabled"""



"""
Packages management
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

from .python_package import PythonPackage
from .install_python_packages import install_python_packages
from .restore_nuget_packages import restore_nuget_packages
from .nuget_package import NugetPackage
from .inno_package import InnoPackage

__all__ = ['PythonPackage', 'install_python_packages', 'restore_nuget_packages', 'NugetPackage',
           'InnoPackage']



"""
Pykrete tools tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging
import unittest
from pykrete.tools import VisualStudio, Externals


class PykreteToolsTestCase(unittest.TestCase):
    """Unit tests pykrete's tools
    """

    _logger = logging.getLogger(__name__)

    def test_visual_studio(self):
        """Verify creation and basic usage of the VisualStudio tool set
        """
        visual_studio = VisualStudio()
        self._logger.info(visual_studio)
        self._logger.info(visual_studio.build.path)
        self._logger.info(visual_studio.build.run('-version', 'failed to get msbuild version'))
        self._logger.info(visual_studio.test.path)
        self._logger.info(visual_studio.test.run('/help', 'failed to get mstest help'))

    def test_externals(self):
        """Verify creation and basic usage of the VisualStudio tool set
        """
        externals = Externals()
        self._logger.info(externals)
        nuget = externals.nuget()
        self._logger.info(nuget.path)
        self._logger.info(nuget.run('help', 'failed to get nuget help'))



"""
Pykrete versioning.formatting tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

import unittest
import logging
from pykrete.versioning import Version, Formatting, RevisionType


class PykreteVersioningFormattingTestCase(unittest.TestCase):
    """Unit tests for pykrete's versioning module's Formatting class
    """

    _logger = logging.getLogger(__name__)

    def test_release_version_formatting(self):
        """Verifies formatting release versions"""
        target = Formatting(Version(1, 2, 4, 3, RevisionType.Release))
        self.assertEqual('1.2.3', target.for_python, 'wrong python format')

    def test_rc_version_formatting(self):
        """Verifies formatting rc versions"""
        target = Formatting(Version(11, 22, 3, 44, RevisionType.RC))
        self.assertEqual('11.22rc44', target.for_python, 'wrong python format')

    def test_beta_version_formatting(self):
        """Verifies formatting beta versions"""
        target = Formatting(Version(12, 23, 2, 34, RevisionType.Beta))
        self.assertEqual('12.23b34', target.for_python, 'wrong python format')

    def test_alpha_version_formatting(self):
        """Verifies formatting alpha versions"""
        target = Formatting(Version(21, 32, 1, 43, RevisionType.Alpha))
        self.assertEqual('21.32a43', target.for_python, 'wrong python format')

    def test_pre_alpha_version_formatting(self):
        """Verifies formatting pre-alpha versions"""
        target = Formatting(Version(111, 222, 0, 333, RevisionType.PreAlpha))
        self.assertEqual('111.222pa333', target.for_python, 'wrong python format')


if __name__ == '__main__':
    unittest.main()



"""
Text replacement class
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from pykrete.io.file.replacement_base import ReplacementBase


class TextReplacement(ReplacementBase):
    """Holds regular-expression text replacement request
    """

    def _do_replace(self, target):
        """Perform this replacement on the specified target

        :param target: (string) replacement target string
        """
        return target.replace(self._find, self._replacement)



"""
Pykrete versioning.ci_version tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

import unittest
import logging
from pykrete.args import CiIo
from pykrete.versioning import CiVersion, RevisionType
from .versioning import PykreteVersioningTestCase


class PykreteVersioningCiVersionTestCase(PykreteVersioningTestCase):
    """Unit tests for pykrete's versioning module's CiVersion class
    """

    _logger = logging.getLogger(__name__)
    _local_release = {
        'major version': '1',
        'minor version': '2',
        'build version': '3',
        'branch name': 'master',
        'merge request title': 'None',
        'job name': 'some_job'}
    _local_rc_master_rc_job = {
        'major version': '11',
        'minor version': '22',
        'build version': '33',
        'branch name': 'master',
        'merge request title': 'None',
        'job name': 'some_rc_job'}
    _local_rc_merge_request = {
        'major version': '12',
        'minor version': '23',
        'build version': '34',
        'branch name': 'custom',
        'merge request title': 'title'}
    _local_beta = {
        'major version': '111',
        'minor version': '222',
        'build version': '333',
        'branch name': 'custom',
        'merge request title': 'WIP: title'}
    _local_alpha = {
        'major version': '123',
        'minor version': '456',
        'build version': '789',
        'branch name': 'custom',
        'merge request title': 'None'}

    def test_ci_version_local_release(self):
        """Verifies handling of simulated CI-environment's release version"""
        self._assert_ci_version_local(
            spec=self._local_release,
            expected_revision_type=RevisionType.Release,
            expected_revision=4)

    def test_ci_version_local_rc_master_rc_job(self):
        """Verifies handling of simulated CI-environment's release-candidate version"""
        self._assert_ci_version_local(
            spec=self._local_rc_master_rc_job,
            expected_revision_type=RevisionType.RC,
            expected_revision=3)

    def test_ci_version_local_rc_merge_request(self):
        """Verifies handling of simulated CI-environment's release-candidate version"""
        self._assert_ci_version_local(
            spec=self._local_rc_merge_request,
            expected_revision_type=RevisionType.RC,
            expected_revision=3)

    def test_ci_version_local_beta(self):
        """Verifies handling of simulated CI-environment's beta version"""
        self._assert_ci_version_local(
            spec=self._local_beta,
            expected_revision_type=RevisionType.Beta,
            expected_revision=2)

    def test_ci_version_local_alpha(self):
        """Verifies handling of simulated CI-environment's alpha version"""
        self._assert_ci_version_local(
            spec=self._local_alpha,
            expected_revision_type=RevisionType.Alpha,
            expected_revision=1)

    def _assert_ci_version_local(self, spec, expected_revision_type, expected_revision):
        target = CiVersion(CiIo(self._echo, spec))
        self._assert_version_pattern(target)
        self._assert_spec(spec, target)
        self.assertEqual(expected_revision_type, target.revision_type, "Wrong revision type")
        self.assertEqual(expected_revision, target.revision, "Wrong revision")

    def _assert_spec(self, spec, target):
        parts = {'major': target.major, 'minor': target.minor, 'build': target.build}
        for key, value in parts.items():
            self.assertTrue(isinstance(value, int), f'{key} is not int')
            self.assertEqual(int(spec[f'{key} version']), value, f'Wrong {key} version')


if __name__ == '__main__':
    unittest.main()



"""
Pykrete setup
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

from .twine_command import TwineCommand
from .build_git_ssh_tag import BuildGitSshTag

__all__ = ['pylint', 'TwineCommand', 'BuildGitSshTag']



"""
StdOut and StdErr log maker
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import logging


class LessThanFilter(logging.Filter):
    """Logging filter for messages below a certain level
    """

    def __init__(self, exclusive_maximum, name=""):
        """Initialize this instance to filter messages below the maximum

        :param exclusive_maximum: Exclusive maximum log level
        :param name: filter name (optional)
        """
        super().__init__(name)
        self.max_level = exclusive_maximum

    def filter(self, record):
        """Filter log records

        :param record: log record
        :return: 1 if record is to be shown, 0 otherwise
        """
        return 1 if record.levelno < self.max_level else 0

    def __str__(self):
        return "LessThan " + self.max_level



"""
Pykrete NuGet tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
import unittest
import os
from pykrete.packages.nuget_package import NugetPackage
from pykrete.io.file import File


class TestNuget(unittest.TestCase):
    """Unit tests for handling NuGet packages
    """
    _base_path = os.path.join('src', 'pykrete', 'tests', 'src')

    def test_update_release_notes(self):
        """Verifies that updating release notes succeeds
        """
        nuspec_path = os.path.join(self._base_path, 'some.nuspec')
        rtf_path = os.path.join(self._base_path, 'Notes.rtf')
        package = NugetPackage(nuspec_path, None)
        package.update_release_notes(rtf_path)
        self.assertTrue('A newt?' in File(nuspec_path).read(),
                        'Release notes not pasted into nuspec')


if __name__ == '__main__':
    unittest.main()



"""
.Net source project types
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""
from enum import Enum


class ProjectType(Enum):
    """Project type enumeration
    """

    UNKNOWN = 0
    LIBRARY = 1
    EXECUTABLE = 2
    UNIT_TEST = 3
    FLOW_TEST = 4



"""
Pykrete repo.Git.git_internals.GitSsh tests
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

import unittest
import logging
from pykrete.repo.git_internals.git_ssh import GitSsh
from .git_repo_testbed import GitUrlsTestbed, WithTestbed


class PythonGitSshTestCase(unittest.TestCase):
    """E2E tests for pykrete's repo module's git_internals.GitSsh class
    """

    _logger = logging.getLogger(__name__)
    _ssh_file = 'some_file'
    _env = WithTestbed()
    _urls = GitUrlsTestbed()
    _args = 'one', 'two', 'three'

    def test_ssh_set_unset(self):
        """Creation and removal of git ssh connection
        """
        target = self._test_set()
        self._test_unset(target)

    def _test_set(self):
        target = GitSsh(self._setter, self._urls, self._ssh_file, '22')
        self.assertTrue(self._urls.ssh_replaced, 'ssh not replaced on set')
        self.assertTrue(self._env.entered and not self._env.exited, 'wrong env state on set')
        return target

    def _test_unset(self, target):
        target.unset(*self._args)
        self.assertEqual([('http', 'ssh')], self._urls.replaced_urls,
                         'wrong/no URLs replaced on unset')
        self.assertTrue(self._env.entered and self._env.exited, 'wrong env state on unset')
        self.assertEqual(self._args, self._env.args, 'wrong args on unser')

    def _setter(self, **kwargs):
        self._logger.debug('setter called with %s', kwargs)
        cmd = 'GIT_SSH_COMMAND'
        self.assertTrue(cmd in kwargs.keys(), 'SSH command not set')
        self.assertTrue(self._ssh_file in kwargs[cmd], 'file not passed to SSH command')
        self._env.__enter__()
        return self._env


if __name__ == '__main__':
    unittest.main()
